<?php

require 'Datablock.php';

/**
 * The Data Descriptor is used to store information about changes in database
 * table cells.
 * 
 * These changes are split into several data blocks, because of the limitations
 * of php as a scripting language in webservers.
 *
 * @category    Ssr
 * @package     Ssr
 * @subpackage  DataDescriptor
 * @version     Release: 1.0
 * @since       Class available since Release 1.0
 * @author      Benjamin Sommer <developer@benjaminsommer.com>
 */
class Ssr_DataDescriptor {

    /**
     * Create and initialize a data descriptor
     * 
     * @param string $name              The nicename for this descriptor
     * @param string $description       A brief description about this descriptor
     * @param string $pathdir           Where to save the descriptor
     * @param Ssr_TableDescriptor $tableDescriptor  A table descriptor; required for valid
     *                                              datablocks
     */
    public function __construct($name, $description, $pathdir, $tableDescriptor) {
        $this->name = $name;
        $this->desc = $description;
        $this->tabledesc = $tableDescriptor;
        $this->id = max(time(), self::$curmaxid + 1);
        if ($pathdir != null) {
            if (!is_dir($pathdir))
                mkdir($pathdir, 0750, true);
            while (file_exists($this->path = "$pathdir{$this->id}.descriptor.xml")) {
                $this->id++;
            }
            if (file_put_contents($this->path, '') === false) {
                throw new Exception("Cannot allocate space for descriptor file `{$this->path}`");
            }
            unlink($this->path);
            self::$curmaxid = $this->id;
        }
        $this->timer_start = microtime(true);
    }

    /**
     * Load and create a previously saved data descriptor from file
     * @param string $filename      Exact path to file location of data descriptor to be loaded
     * @return Ssr_DataDescriptor   The loaded descriptor, or an empty one if loading fails
     */
    public static function createFromFile($filename) {
        $t = new Ssr_DataDescriptor('', '', null, null);
        $t->path = $filename;
        $t->parse($filename);
        return $t;
    }

    /**
     * Creates a datablock
     * @return Ssr_Datablock 
     */
    public function createDatablock() {
        $id = sizeof($this->blocks);
        $filename = "$this->id.$id.datablock.xml";
        $path = dirname($this->path) . '/' . $filename;
        if (file_put_contents($path, '') === false) {
            throw new Exception("Cannot allocate space for datablock `$path`.");
        }
        $t['id'] = $id;
        $t['path'] = $filename;
        $t['cells'] = 0;
        $t['size'] = 0;
        $this->blocks[] = $t;
        $block = new Ssr_Datablock($path, $this->tabledesc, $this->blocksize);
        if ($this->storeInMem) {
            $this->storedBlocks[] = $block;
            $block->storeInMemory(true);
        }
        return $block;
    }

    /**
     * Updates some additional information about datablocks
     * @param Ssr_Datablock $datablock The datablock being previously create with createDatablock()
     */
    public function updateFromDatablock($datablock) {
        $path = str_ireplace(dirname($this->path) . '/', '', $datablock->getPath());
        foreach ($this->blocks as &$b) {
            if ($b['path'] == $path) {
                $b['cells'] = $datablock->countCells();
                $b['size'] = filesize($datablock->getPath());
            }
        }
    }

    /**
     * Save this datablock to previously specified path. 
     * 
     * Filename has been automatically generated by the constructor to be unique.
     * The method will always return true if blocks are to be stored in memory, and 
     * not saved to drive.
     * 
     * @return bool True on success 
     */
    public function save() {
        if ($this->storeInMem)
            return true;
        if ($this->countCells() == 0 && sizeof($this->blocks) == 0)
            return true;

        $xml = new SimpleXMLElement('<ssrdef/>');
        $xml->addAttribute('version', '1.0');
        $xml->addAttribute('date', date(DateTime::ISO8601));
        $xml->addAttribute('guid', $this->id);
        $xml->addAttribute('name', $this->name);
        $xsum = $xml->addChild('summary');
        $xsum->addAttribute('cells', $this->countCells());
        $xml->addChild('description', $this->desc);
        $xml->addChild('report', $this->reportId);
        $this->tabledesc->asXML($xml);

        $xsel = $xml->addChild('selectors');
        foreach ($this->selectors as $selector)
            $selector->asXML($xsel);

        $matches = 0;
        $xflt = $xml->addChild('filters');
        foreach ($this->filters as $filter) {
            $filter->asXML($xflt);
            $matches += $filter->getTotalMatches();
        }
        $xblocks = $xml->addChild('blocks');
        foreach ($this->blocks as $block) {
            $xblock = $xblocks->addChild('block');
            foreach ($block as $k => $v)
                $xblock->addAttribute($k, $v);
        }
        $this->cputime = microtime(true) - $this->timer_start;
        $xsum->addAttribute('matches', $matches);
        $xsum->addAttribute('cputime', $this->cputime);

        if (!$xml->asXML($this->path)) {
            Ssr_Error::push(400, "Cannot save data descriptor to `{$this->path}`");
            unlink($this->path);
            return false;
        }
        return true;
    }

    /**
     * Try to parse a stored data descriptor.
     * @param string $filename Exact file location where data descriptor is stored
     */
    private function parse($filename) {
        $xml = new SimpleXMLElement(file_get_contents($filename));
        if (isset($xml['name']))
            $this->name = (string) $xml['name'];
        if (isset($xml['date']))
            $this->date = (string) $xml['date'];
        if (isset($xml['version']))
            $this->version = (string) $xml['version'];
        if (isset($xml['guid']))
            $this->id = (int) $xml['guid'];
        if (isset($xml->summary)) {
            $sum = $xml->summary;
            if (isset($sum['cputime']))
                $this->cputime = (string) $sum['cputime'];
        }
        if (isset($xml->description))
            $this->desc = (string) $xml->description;
        if (isset($xml->report))
            $this->reportId = (string) $xml->report;
        $this->tabledesc = Ssr_TableDescriptor::parseXML($xml);
        if (isset($xml->selectors) && $xml->selectors->selector) {
            foreach ($xml->selectors->selector as $xselector) {
                $z->selector = $xselector;
                $t = Ssr_Selector::parseXML($z);
                if ($t)
                    $this->selectors[] = $t;
            }
        }
        $z = null;
        if (isset($xml->filters) && $xml->filters->filter) {
            foreach ($xml->filters->filter as $xfilter) {
                $z->filter = $xfilter;
                $t = Ssr_Filter::parseXML($z);
                if ($t)
                    $this->filters[] = $t;
            }
        }
        if (isset($xml->blocks) && $xml->blocks->block) {
            foreach ($xml->blocks->block as $block) {
                $t = array();
                $t['cells'] = 0;
                $t['id'] = -1;
                $t['size'] = 0;
                $t['path'] = '';
                foreach ($t as $k => $v) {
                    if (isset($block[$k]))
                        $t[$k] = (string) $block[$k];
                }
                $this->blocks[] = $t;
            }
        }
    }

    /**
     * Get the nicename being given for this descriptor
     * @return string Name for this descriptor
     */
    public function getName() {
        return $this->name;
    }

    /**
     * Set the nicename for this descriptor
     * @param string $name A new (unique) nicename.
     */
    public function setName($name) {
        $this->name = $name;
    }

    /**
     * Get the description being given for this descriptor
     * @return string Description for this descriptor
     */
    public function getDescription() {
        return $this->desc;
    }

    /**
     * Set a brief description for this descriptor
     * @param string $desc A brief description about this descriptor; why it was created;
     *                  kind of operation (intend) etc.
     */
    public function setDescription($desc) {
        $this->desc = $desc;
    }

    /**
     * Prepare this descriptor for input. 
     * 
     * Calling this method is optional. Currently only sets the internal timer
     * for additional bookkeeping.
     */
    public function prepare() {
        $this->timer_start = microtime(true);
    }

    /**
     * Get the table descriptor being used by this data descriptor.
     * 
     * Required to identify cells in datablock and to increase safety in backups/restore operations.
     * 
     * @return Ssr_TableDescriptor  
     */
    public function getTableDescriptor() {
        return $this->tabledesc;
    }

    /**
     * Grap all required information from table filter structure
     * 
     * Required to call this method before calling save(), otherwise results in
     * corrupt descriptors.
     * 
     * @param Ssr_TableFilter $tableFilter 
     */
    public function readTableFilter($tableFilter) {
        $this->selectors = $tableFilter->getSelectors();
        $this->filters = $tableFilter->getFilters();
        $this->tabledesc = $tableFilter->getTableDescriptor();
    }

    /**
     * Get an array of all blocks being attached to this data descriptor
     * @return array An array with block parameters; keys and values are identical to
     *              those used in data descriptor definition standard for blocks.
     */
    public function getBlocks() {
        return $this->blocks;
    }

    /**
     * Get the path to given block.
     * 
     * Method is required to resolve location of data blocks, because in the data descriptor,
     * paths to child data blocks are stored relative to the descriptor.
     * 
     * @param int $index
     * @return string       The absolute or relative path to given block
     */
    public function getBlockPath($index) {
        return dirname($this->path) . '/' . $this->blocks[$index]['path'];
    }

    /**
     * Count the number of all blocks being linked from this descriptor.
     * 
     * @return int Number of total child blocks
     */
    public function countBlocks() {
        return sizeof($this->blocks);
    }

    /**
     * Get an already loaded block object.
     * 
     * This method either tries to initialize a block object from hard drive, or 
     * returns the block object hold in memory is blocks are to be stored in memory 
     * for this instance.
     * 
     * @param type $index
     * @return type 
     */
    public function getBlock($index) {
        if ($this->storeInMem)
            return $this->storedBlocks[$index];
        else
            return Ssr_Datablock::createFromFile($this->getBlockPath($index));
    }

    /**
     * Get the path or file location for this descriptor.
     * @return string
     */
    public function getPath() {
        return $this->path;
    }

    /**
     * Get a unique id for this descriptor.
     * @return int
     */
    public function getId() {
        return $this->id;
    }

    /**
     * Get the total number of cells in all blocks being linked from this descriptor.
     * @return int
     */
    public function countCells() {
        $c = 0;
        foreach ($this->blocks as $block)
            $c+=$block['cells'];
        return $c;
    }

    /**
     * Get the filesize of this descriptor and all child blocks
     * @return int
     */
    public function getSize() {
        $s = 0;
        foreach ($this->blocks as $block)
            $s+=$block['size'];
        if (file_exists($this->path))
            $s+=filesize($this->path);
        return $s;
    }

    /**
     * Get the total number of selectors
     * @return int
     */
    public function countSelectors() {
        return sizeof($this->selectors);
    }

    /**
     * Get an selector object by given index.
     * @param int $index
     * @return Ssr_Selector 
     */
    public function getSelector($index) {
        return $this->selectors[$index];
    }

    /**
     * Get the total number of filters.
     * @return int
     */
    public function countFilter() {
        return sizeof($this->filters);
    }

    /**
     * Copy all selectors from another data descriptor to this descriptor.
     * @param Ssr_DataDescriptor $datadesc 
     */
    public function copySelectors($datadesc) {
        $this->selectors = $datadesc->selectors;
    }

    /**
     * Copy all filters from another data descriptor to this descriptor.
     * @param Ssr_DataDescriptor $datadesc 
     */
    public function copyFilter($datadesc) {
        $this->filters = $datadesc->filters;
    }

    /**
     * Get a filter object for given index
     * @param int $index
     * @return Ssr_Filter
     */
    public function getFilter($index) {
        return $this->filters[$index];
    }

    /**
     * Get current/used blocksize for all child blocks
     * @return int
     */
    public function getBlocksize() {
        return $this->blocksize;
    }

    /**
     * Update the blocksize for all newly created blocks.
     * @param int $blocksize  Should be between 100 and 1000.
     */
    public function setBlocksize($blocksize) {
        if ($blocksize > 0)
            $this->blocksize = $blocksize;
    }

    /**
     * Get the unique report name as an additional attribute
     * @return string 
     */
    public function getReportName() {
        return $this->reportId;
    }

    /**
     * Set the report name for this data descriptor. Used by Ssr_Execute.
     * @param string $reportId 
     */
    public function setReportName($reportId) {
        $this->reportId = $reportId;
    }

    /**
     * Whether to store newly created blocks in memory, rather then saving them to hard drive.
     * 
     * Quite handy to simulate operations or to get a preview.
     * 
     * @param bool $b 
     */
    public function storeInMemory($b) {
        $this->storeInMem = $b;
    }

    private $name = '';
    private $date = '';
    private $version = '';
    private $desc = '';
    private $cputime = '';
    private $tabledesc = null;
    private $selectors = array();
    private $filters = array();
    private $blocks = array();
    private $reportId = '';
    private $storeInMem = false;
    private $storedBlocks = array();
    private $path = '';
    private $id = 0;
    private $timer_start = 0;
    private $blocksize = 1000;
    private static $curmaxid = 0;

}

?>
