<?php

/**
 * The datablock actually stores all changes of table cells. If parent data descriptor
 * is corrupt or lost, this datablock can still be used to restore previous operations.
 *
 * @category    Ssr
 * @package     Ssr
 * @subpackage  Datablock
 * @version     Release: 1.0
 * @since       Class available since Release 1.0
 * @author      Benjamin Sommer <developer@benjaminsommer.com>
 */
class Ssr_Datablock {

    /**
     * Creates a new datablock
     * @param string $file  Where to save this block
     * @param Ssr_TableDescriptor $tableDescriptor Must equal the one in its parent
     *                          data descriptor.
     * @param int $blocksize The number of cells to be stored in this file.
     */
    public function __construct($file, $tableDescriptor, $blocksize=1000) {
        $this->path = $file;
        $this->tabledesc = $tableDescriptor;
        $this->blocksize = $blocksize;
    }

    /**
     * Try to load a data block from drive.
     * @param string $file
     * @return Ssr_Datablock Loaded block; an empty block if fails
     */
    public static function createFromFile($file) {
        $t = new Ssr_Datablock($file, null);
        $t->parse();
        return $t;
    }

    /**
     * Push a table cell to this data block.
     * @param string $rowId         The value for the primary column for this table row.
     *                              Needed to later identify a table cell
     * @param string $columnName    The column name for this cell
     * @param string $preContent    The content before changes/table filter
     * @param string $postContent   The content of this cell after modification
     */
    public function pushCell($rowId, $columnName, $preContent, $postContent) {
        $cell = array();
        $cell['row'] = $rowId;
        $cell['col'] = $columnName;
        $cell['pre'] = $preContent;
        $cell['post'] = $postContent;
        $this->cells[] = $cell;
    }

    /**
     * Check if data block has enough space for another cell
     * @return bool
     */
    public function hasSpace() {
        return sizeof($this->cells) < $this->blocksize;
    }

    /**
     * Save this block to hard drive (only when not to store block in memory)
     * @return bool
     */
    public function save() {
        if ($this->storeInMem)
            return true;

        $xml = new SimpleXMLElement('<ssrdata/>');
        $xml->addAttribute('storage', 'bycells');
        $xml->addAttribute('blocksize', $this->blocksize);
        $xml->addAttribute('version', '1.0');
        $xml->addAttribute('date', date(DateTime::ISO8601));
        $this->tabledesc->asXML($xml);

        $xcells = $xml->addChild('cells');
        foreach ($this->cells as $cell) {
            $xcell = $xcells->addChild('cell');
            $xcell->addAttribute('col', $cell['col']);
            $xcell->addAttribute('row', $cell['row']);
            $xcell->pre = $cell['pre'];
            $xcell->post = $cell['post'];
        }
        $xcells->addAttribute('count', sizeof($this->cells));
        if (!$xml->asXML($this->path)) {
            Ssr_Error::push(400, "Cannot save datablock `{$this->path}`");
            return false;
        }
        return true;
    }

    /**
     * Try to parse this block from file
     */
    private function parse() {
        $xml = new SimpleXMLElement(file_get_contents($this->path));
        if (isset($xml['date']))
            $this->date = (string) $xml['date'];
        if (isset($xml['version']))
            $this->version = (string) $xml['version'];
        if (isset($xml['blocksize']))
            $this->blocksize = (int) $xml['blocksize'];
        if (isset($xml['storage']))
            $this->storage = (string) $xml['storage'];
        $this->tabledesc = Ssr_TableDescriptor::parseXML($xml);
        if (isset($xml->cells) && isset($xml->cells['count']) && isset($xml->cells->cell)) {
            $count = (int) $xml->cells['count'];
            foreach ($xml->cells->cell as $c) {
                if (!isset($c['row']) || !isset($c['col']) || !isset($c->pre) || !isset($c->post))
                    continue;
                $t = array();
                $t['row'] = (string) $c['row'];
                $t['col'] = (string) $c['col'];
                $t['pre'] = (string) $c->pre;
                $t['post'] = (string) $c->post;
                $this->cells[] = $t;
            }
            $tc = sizeof($this->cells);
            if ($tc != $count)
                Ssr_Warning::push(400, "Missing data in `{$this->path}` (counted " . ($tc - $count) . ")");
        }
    }

    /**
     * Check if given cell matches the current entry in the database.
     * @param mysqli $db 
     */
    public function cellMatchesDatabase($cellIndex, $db, $direction=1) {
        $cell = $this->cells[$cellIndex];
        if ($direction == 1)
            $from = $cell['pre'];
        else
            $from = $cell['post'];

        $query = "SELECT `{$cell['col']}`
                  FROM `{$this->tabledesc->getDatabaseName()}`.`{$this->tabledesc->getTableName()}`
                  WHERE `{$this->tabledesc->getPrimaryColumnName()}` = '{$cell['row']}' 
                    AND `{$cell['col']}` = '" . $db->real_escape_string($from) . "'" . PHP_EOL;
        if (($result = $db->query($query))) {
            $rows = $result->num_rows;
            $result->close();
            return $rows > 0;
        } else {
            return false;
        }
    }

    /**
     * Get full path to this block on drive
     * @return string
     */
    public function getPath() {
        return $this->path;
    }

    /**
     * Count the number of cells in this block
     * @return int
     */
    public function countCells() {
        return sizeof($this->cells);
    }

    /**
     * Get an array of attributes and values for given cell
     * @param int  $index   The index for this cell, starting with 0.
     * @return array        An array with keys and values as defined in the 
     *                      xml standard definition for these data blocks
     */
    public function getCell($index) {
        return $this->cells[$index];
    }

    /**
     * Get how cells are stored in this file
     * @return string
     */
    public function getStorageType() {
        return $this->storage;
    }

    /**
     * Set how cells are to be stored in this fill
     * @param string $storage Currently, only 'bycells' is implemented
     */
    public function setStorageType($storage) {
        $this->storage = $storage;
    }

    /**
     * Get the underlying table descriptor
     * @return Ssr_TableDescriptor 
     */
    public function getTableDescriptor() {
        return $this->tabledesc;
    }

    /**
     * Get the maximum number of cells to be stored in this block
     * @return int
     */
    public function getBlocksize() {
        return $this->blocksize;
    }

    /**
     * Get the version info parsed from file.
     * @return string
     */
    public function getVersion() {
        return $this->version;
    }

    /**
     * Get the date when this block has been saved/created
     * @return string
     */
    public function getDate() {
        return $this->date;
    }

    /**
     * Whether to store this block in memory or to allow it to be saved to drive
     * @param bool $b 
     */
    public function storeInMemory($b) {
        $this->storeInMem = $b;
    }

    /**
     *
     * @var Ssr_TableDescriptor
     */
    private $tabledesc = null;
    private $path = '';
    private $blocksize = 1000;
    private $storeInMem = false;
    private $version = '1.0';
    private $storage = 'bycells';
    private $date = '';
    private $cells = array();

}

?>
