<?php

/**
 * An instance of Execute is used to actually change the database table.
 * 
 * The data comes from given data descriptor and thus its data blocks.
 * When executing, reports and new data descriptors are generated, one descriptor
 * for ok and one for failed changes.
 *
 * @category    Ssr
 * @package     Ssr
 * @subpackage  Execute
 * @version     Release: 1.0
 * @since       Class available since Release 1.0
 * @author      Benjamin Sommer <developer@benjaminsommer.com>
 * @uses        Ssr_DataDescriptor
 */
class Ssr_Execute {

    /**
     * Create a executioner to actually change the database
     * @param Ssr_DataDescriptor $dataDescriptor
     * @param mysqli $db 
     */
    public function __construct($dataDescriptor, $db, $pathdir, $reportDir) {
        $this->db = $db;
        $this->datadesc = $dataDescriptor;
        $tabledesc = $dataDescriptor->getTableDescriptor();
        $this->datadesc_ok = new Ssr_DataDescriptor('Report OK', $dataDescriptor->getDescription(), $pathdir, $tabledesc);
        $this->datadesc_ok->copyFilter($dataDescriptor);
        $this->datadesc_ok->copySelectors($dataDescriptor);
        $this->datadesc_fail = new Ssr_DataDescriptor('Report Fail', $dataDescriptor->getDescription(), $pathdir, $tabledesc);
        $this->datadesc_fail->copyFilter($dataDescriptor);
        $this->datadesc_fail->copySelectors($dataDescriptor);
        $this->reportDir = $reportDir;
    }

    /**
     * Execute all changes for the given block id in current data descriptor
     * @param int $blockId   The block index id, starting with 0
     * @param int $direction forward run := 1; backward run := -1 
     */
    public function exec($blockId, $direction=1) {
        ini_set('max_execution_time', 5 * 60);

        $path = $this->datadesc->getPath();
        $this->datablock_fail = $this->datadesc_fail->createDatablock();
        $this->datablock_ok = $this->datadesc_ok->createDatablock();

        $file = $this->datadesc->getBlockPath($blockId);
        if (file_exists($file)) {
            $datablock = Ssr_Datablock::createFromFile($file);
            $this->expandReport("Running Datablock #$blockId " . ($direction == 1 ? 'forward' : 'backward'));
            $this->run($datablock, $direction);
        } else {
            Ssr_Error::push(400, "Missing datablock `$file` in data descriptor `{$this->datadesc->getPath()}`");
        }

        if ($this->datablock_fail->countCells() > 0) {
            $this->datablock_fail->save();
            $this->datadesc_fail->updateFromDatablock($this->datablock_fail);
            $this->expandReport("Child Fail Datablock created at {$this->datablock_fail->getPath()}");
        }
        if ($this->datablock_ok->countCells() > 0) {
            $this->datablock_ok->save();
            $this->datadesc_ok->updateFromDatablock($this->datablock_ok);
            $this->expandReport("Child OK Datablock created at {$this->datablock_ok->getPath()}");
        }
        $this->updateDataDescriptorInfos($blockId);
        $reportFname = basename($this->saveReport());
        if ($this->datadesc_fail->countCells() > 0) {
            $this->datadesc_fail->setReportName($reportFname);
            $this->datadesc_fail->save();
            $this->expandReport("Child Fail DataDescriptor created at {$this->datadesc_fail->getPath()}");
        }
        if ($this->datadesc_ok->countCells() > 0) {
            $this->datadesc_ok->setReportName($reportFname);
            $this->datadesc_ok->save();
            $this->expandReport("Child OK DataDescriptor created at {$this->datadesc_ok->getPath()}");
        }
    }

    /**
     * Update database from datablock
     * @param Ssr_Datablock $datablock
     * @param int           $direction  forward run := 1; backward run := -1 
     */
    private function run($datablock, $direction=1) {
        $td = $datablock->getTableDescriptor();
        $query = "UPDATE `{$td->getDatabaseName()}`.`{$td->getTableName()}` ";
        $primaryCol = $td->getPrimaryColumnName();
        $max = $datablock->countCells();
        for ($i = 0; $i < $max; $i++) {
            $cell = $datablock->getCell($i);
            $rowId = $cell['row'];
            $columnName = $cell['col'];
            if ($direction == -1) {
                $from = $cell['post'];
                $to = $cell['pre'];
            } else {
                $from = $cell['pre'];
                $to = $cell['post'];
            }
            $q = $query . "
                SET `$columnName` = '" . $this->db->real_escape_string($to) . "' 
                WHERE `$primaryCol` = '$rowId' 
                 AND `$columnName` = '" . $this->db->real_escape_string($from) . "'" . PHP_EOL;

            if ($this->db->query($q) && $this->db->affected_rows > 0) {
                if (!$this->datablock_ok->hasSpace()) {
                    $this->datablock_ok->save();
                    $this->datadesc_ok->updateFromDatablock($this->datablock_ok);
                    $this->expandReport("Child OK Datablock created at {$this->datablock_ok->getPath()}");
                    $this->datablock_ok = $this->datadesc_ok->createDatablock();
                }
                $this->datablock_ok->pushCell($rowId, $columnName, $from, $to);
            } else {
                if (!$this->datablock_fail->hasSpace()) {
                    $this->datablock_fail->save();
                    $this->datablock_fail = $this->datadesc_fail->createDatablock();
                    $this->expandReport("Child Fail Datablock created at {$this->datablock_fail->getPath()}");
                    $this->datadesc_fail->updateFromDatablock($this->datablock_fail);
                }
                $this->datablock_fail->pushCell($rowId, $columnName, $from, $to);
                if ($this->db->errno == 0)
                    $this->expandReport("r$rowId:$columnName version conflict");
                else
                    $this->expandReport("r$rowId:$columnName {$this->db->errno}:{$this->db->error}");
            }
        }
    }

    /**
     * A small helper function to update data descriptor created for this execution
     * @param int $blockId 
     */
    private function updateDataDescriptorInfos($blockId) {
        $count_ok = $this->datadesc_ok->countCells();
        $count_fail = $this->datadesc_fail->countCells();
        $count_total = $count_ok + $count_fail;
        $this->datadesc_ok->setName("Executed $count_ok/$count_total elements from `{$this->datadesc->getName()}`::$blockId::OK");
        $this->datadesc_fail->setName("Executed $count_ok/$count_total elements from `{$this->datadesc->getName()}`::$blockId::Error");
        $desc = "Execution Report from " . date(DateTime::ISO8601) . "" . PHP_EOL .
                "Source Data Descriptor: {$this->datadesc->getName()}" . PHP_EOL .
                "Source Data Descriptor Path: {$this->datadesc->getPath()}" . PHP_EOL .
                "Number of Data Blocks available: {$this->datadesc->countBlocks()}" . PHP_EOL .
                "Number of Data Cells available: {$this->datadesc->countCells()}" . PHP_EOL .
                "" . PHP_EOL;
        $this->report = $desc . PHP_EOL . $this->report;
    }

    /**
     * Save the generated report during execution to harddrive and return its location
     * @return file New report location
     */
    private function saveReport() {
        $report_dir = $this->reportDir . $this->datadesc->getId();
        $report_num = 1;
        do {
            $report_filename = "$report_dir-$report_num.report";
            $report_num++;
        } while (file_exists($report_filename));
        file_put_contents($report_filename, $this->report);
        $this->reportPath = $report_filename;
        return $report_filename;
    }

    /**
     * A helper method to store something in the report
     * @param string $message 
     */
    private function expandReport($message) {
        $this->report .= $message . PHP_EOL;
    }

    /**
     * Get the full path to the current report location
     * @return string 
     */
    public function getReportPath() {
        return $this->reportPath;
    }

    /**
     * Get the number of cells being executed without errors
     * @return int 
     */
    public function countCellsOk() {
        return $this->datadesc_ok->countCells();
    }

    /**
     * Get the number of cells for which execution failed
     * @return int 
     */
    public function countCellsFail() {
        return $this->datadesc_fail->countCells();
    }

    /**
     * @var mysqli
     */
    private $db;

    /**
     * @var Ssr_DataDescriptor
     */
    private $datadesc = null;

    /**
     * @var Ssr_DataDescriptor 
     */
    private $datadesc_ok = null;

    /**
     * @var Ssr_DataDescriptor 
     */
    private $datadesc_fail = null;

    /**
     * @var Ssr_Datablock
     */
    private $datablock_ok = null;

    /**
     * @var Ssr_Datablock
     */
    private $datablock_fail = null;
    private $report = '';
    private $reportDir = '';
    private $reportPath = '';

}

?>
