<?php

/**
 * A simple filter automatically recognizing string or regex replacements.
 *
 * @category    Ssr
 * @package     Ssr
 * @subpackage  Filter
 * @version     Release: 1.0
 * @since       Class available since Release 1.0
 * @author      Benjamin Sommer <developer@benjaminsommer.com>
 */
class Ssr_Filter {

    /**
     * Initialize a new filter
     * @param string $column    The table column where to apply this filter. 
     *                          Used because an object is given this filter (directly from query)
     *                          Enables more abstraction.
     * @param string $search_pattern    Either a string or valid regex search pattern
     * @param string $replace_pattern   Either a string or valid regex replace pattern
     */
    public function __construct($column, $search_pattern, $replace_pattern) {
        $this->column = $column;
        $this->search_pattern = $search_pattern;
        $this->replace_pattern = $replace_pattern;
        if (strlen($search_pattern) > 1)
            $this->isRegex = ($search_pattern[0] == '/' && $search_pattern[1] != '/');
    }

    /**
     * Filter a result object returned from a database query via mysqli
     * @param tableObject $obj
     * @return int      The number of matches 
     */
    public function filter($obj) {
        $str = $this->getColumnValue($obj);
        $count = 0;
        if ($this->isRegex) {
            $this->result = @preg_replace($this->search_pattern, $this->replace_pattern, $str, -1, $count);
            if (!$this->result)
                Ssr_Error::push(400, "Invalid filter `{$this->search_pattern}`=>`{$this->replace_pattern}`");
        } else {
            $this->result = str_ireplace($this->search_pattern, $this->replace_pattern, $str, $count);
        }
        $this->total_matches += $count;
        return $count > 0;
    }

    /**
     * Get the current/original column value before this filter is applied to.
     * @param tableObject $obj
     * @return string           The column value or null if invalid input object 
     */
    public function getColumnValue($obj) {
        $col = $this->column;
        if (isset($obj->$col))
            return $obj->$col;
        return null;
    }

    /**
     * Get the column name on which this filter will be applied to
     * @return string 
     */
    public function getColumnName() {
        return $this->column;
    }

    /**
     * Get current search pattern
     * @return string 
     */
    public function getSearchPattern() {
        return $this->search_pattern;
    }

    /**
     * Get current replace pattern
     * @return string
     */
    public function getReplacePattern() {
        return $this->replace_pattern;
    }

    /**
     * Check if filter uses regex functionality
     * @return bool
     */
    public function isRegex() {
        return $this->isRegex;
    }

    /**
     * Get the result of the filtering process after calling the filter() function.
     * @return string The cell value after filtering it
     */
    public function getResult() {
        return $this->result;
    }

    /**
     * Get the number of total machtes.
     * 
     * One filter instance can be applied to many table rows.
     * 
     * @return int
     */
    public function getTotalMatches() {
        return $this->total_matches;
    }

    /**
     * Attach properties from this instance to given XML structure
     * @param SimpleXMLElement $xmlElement 
     */
    public function asXML($xmlElement) {
        $x = $xmlElement->addChild('filter');
        $x->addAttribute('column', $this->column);
        $x->addAttribute('search', $this->search_pattern);
        $x->addAttribute('replace', $this->replace_pattern);
        $x->addAttribute('matches', $this->total_matches);
    }

    /**
     * Try to parse an xml element and return an Ssr_Filter
     * @param SimpleXMLElement $xmlElement
     * @return Ssr_Filter 
     */
    public static function parseXML($xmlElement) {
        if (isset($xmlElement->filter)) {
            $f = $xmlElement->filter;
            if (isset($f['column']) && isset($f['search']) && isset($f['replace']) && isset($f['matches'])) {
                $col = (string) $f['column'];
                $s = (string) $f['search'];
                $r = (string) $f['replace'];
                $m = (int) $f['matches'];
                $t = new Ssr_Filter($col, $s, $r);
                $t->total_matches = $m;
                return $t;
            }
        }
        return null;
    }

    private $column;
    private $search_pattern;
    private $replace_pattern;
    private $isRegex;
    private $result;
    private $total_matches = 0;

}

?>
