<?php

/**
 * A simple selector automatically recognizing string or regex patterns.
 *
 * @category    Ssr
 * @package     Ssr
 * @subpackage  Selector
 * @version     Release: 1.0
 * @since       Class available since Release 1.0
 * @author      Benjamin Sommer <developer@benjaminsommer.com>
 */
class Ssr_Selector {

    /**
     * Create a new selector. Used to narrow possible results from database tables.
     * @param string $column The table column name where to apply this selector
     * @param string $pattern The selection pattern
     */
    public function __construct($column, $pattern) {
        $this->column = $column;
        $this->pattern = $pattern;
        if (strlen($pattern) > 1)
            $this->isRegex = ($pattern[0] == '/' && $pattern[1] != '/');
        $this->highlight_pattern = null;
    }

    /**
     * Check whether the table row return by mysqli query matches the selector
     * @param tableRowObject $obj The row returned by mysqli fetch object for given database table.
     * @return bool 
     */
    public function matches($obj) {
        $col = $this->column;
        $str = $obj->$col;
        if ($this->isRegex) {
            return (bool) preg_match($this->pattern, $str);
        } else {
            return stripos($str, $this->pattern) !== false;
        }
    }

    /**
     * Get the column name where to apply this selector
     * @return string
     */
    public function getColumnName() {
        return $this->column;
    }

    /**
     * Get the selection pattern
     * @return string
     */
    public function getPattern() {
        return $this->pattern;
    }

    /**
     * Check whether selection pattern is regex or literal
     * @return bool 
     */
    public function isRegexp() {
        return $this->isRegex;
    }

    /**
     * Attach properties from this instance to given XML structure
     * @param SimpleXMLElement $xmlElement 
     */
    public function asXML($xmlElement) {
        $x = $xmlElement->addChild('selector');
        $x->addAttribute('column', $this->column);
        $x->addAttribute('pattern', $this->pattern);
    }

    /**
     * Try to parse an xml element and return an Selector
     * @param SimpleXMLElement $xmlElement 
     * @return Ssr_Selector
     */
    public static function parseXML($xmlElement) {
        if (isset($xmlElement->selector)) {
            if (isset($xmlElement->selector['column']) &&
                    isset($xmlElement->selector['pattern'])) {
                $col = (string) $xmlElement->selector['column'];
                $pat = (string) $xmlElement->selector['pattern'];
                if (!empty($col) && !empty($pat))
                    return new Ssr_selector($col, $pat);
            }
        }
        return null;
    }

    private $column;
    private $pattern;
    private $isRegex;

}

?>
