<?php

/**
 * A table descriptor is used to store minimal information about a given table.
 * 
 * This is used by many classes in this package. Used for safety reasons.
 *
 * @category    Ssr
 * @package     Ssr
 * @subpackage  TableDescriptor
 * @version     Release: 1.0
 * @since       Class available since Release 1.0
 * @author      Benjamin Sommer <developer@benjaminsommer.com>
 */
class Ssr_TableDescriptor {

    /**
     * Initialize a table descriptor from database
     * 
     * @param string $name          The table name
     * @param mysqli $db            The current mysqli connection
     * @param string $database      Database name to search for the table
     */
    public function __construct($name, $db, $database='') {
        $this->name = $name;
        $this->database = $database;
        if ($db)
            $this->loadColumns($db);
    }

    /**
     * Load column information for this table 
     * @param mysqli $db 
     */
    private function loadColumns($db) {
        if (!empty($this->database))
            $q = "`{$this->database}`.`{$this->name}`";
        else
            $q = "`{$this->name}`";
        $query = "DESCRIBE $q";

        $result = $db->query($query);
        if ($result) {
            while ($obj = $result->fetch_object()) {
                $this->cols[] = $obj;
            }
        } else {
            Ssr_Error::push($db->errno, $db->error);
            return false;
        }
        $result->close();

        $index = $this->findPrimaryColumn('PRI');
        if ($index === null)
            $index = $this->findPrimaryColumn('UNI');
        if ($index !== null) {
            $this->primary_column = $this->getColumnName($index);
        } else {
            Ssr_Error::push(400, "Missing primary column in $q");
            if (defined('DEBUG'))
                var_dump($this->cols);
            return false;
        }
        return true;
    }

    /**
     * Get value of primary key for given resultset
     * @param mysqliobject $obj
     * @return string 
     */
    public function getResultsetKey($obj) {
        $col = $this->primary_column;
        if (isset($obj->$col))
            return $obj->$col;
        return null;
    }

    /**
     * Append this instance to an XML element
     * 
     * @param SimpleXMLElement $xmlElement 
     */
    public function asXML($xmlElement) {
        $xdef = $xmlElement->addChild('tabledef');
        $xdef->addAttribute('database', $this->database);
        $xdef->addAttribute('table', $this->name);
        $xdef->addAttribute('primarycolumn', $this->getPrimaryColumnName());
        foreach ($this->cols as $id => $obj) {
            $col = $xdef->addChild('column');
            $col->addAttribute('name', $obj->Field);
            if (!empty($obj->Key))
                $col->addAttribute('key', $obj->Key);
        }
    }

    /**
     * Try to parse an xml element and return this object
     * @param SimpleXMLElement $xmlElement
     * @return Ssr_TableDescriptor 
     */
    public static function parseXML($xmlElement) {
        if (isset($xmlElement->tabledef)) {
            $d = $xmlElement->tabledef;
            if (isset($d['database']) && isset($d['table']) && isset($d['primarycolumn']) && isset($d->column)) {
                $x = new Ssr_TableDescriptor((string) $d['table'], null, (string) $d['database']);
                $x->primary_column = (string) $d['primarycolumn'];
                foreach ($d->column as $column) {
                    if (isset($column['name'])) {
                        $c = new stdClass();
                        $c->Field = (string) $column['name'];
                        $c->Key = '';
                        if (isset($column['key']))
                            $c->Key = (string) $column['key'];
                        $x->cols[] = $c;
                    }
                }
                return $x;
            }
        }
        return null;
    }

    /**
     * Get the number of columns for this table
     * @return int
     */
    public function size() {
        return sizeof($this->cols);
    }

    /**
     * Get column name for given index
     * @param   int     $index  The numerical column number; 
     *                          with [0 <= $index < size()]
     * @return  string          The column name 
     */
    public function getColumnName($index) {
        return $this->cols[$index]->Field;
    }

    /**
     * Get column key for given index
     * 
     * @param   int     $index  The numerical column number; 
     *                          with [0 <= $index < size()].
     * @return  string          The column key, whereas 'PRI' = primary key, 
     *                          'UNI' = unique key.
     */
    public function getColumnKey($index) {
        return $this->cols[$index]->Key;
    }

    /**
     * Get column number for given column name
     * 
     * @param string $columnName
     * @return int      Returns -1 if name not found
     */
    public function getColumnNumber($columnName) {
        foreach ($this->cols as $i => $o)
            if ($o->Field == $columnName)
                return $i;
        return -1;
    }

    /**
     * Try to find a column with an indexing key, like primary or unique key.
     * @param string    $key    A value representing the key: PRI = primary key,
     *                          UNI = unique key.
     * @return int              The index for matched column, or NULL.
     */
    private function findPrimaryColumn($key='PRI') {
        foreach ($this->cols as $index => $obj) {
            if ($obj->Key == $key)
                return $index;
        }
        return null;
    }

    /**
     * Get the column with a primary, unique index.
     * @return string
     */
    public function getPrimaryColumnName() {
        return $this->primary_column;
    }

    /**
     * Get the database name for this table
     * @return string
     */
    public function getDatabaseName() {
        return $this->database;
    }

    /**
     * Get the table name
     * @return string
     */
    public function getTableName() {
        return $this->name;
    }

    /**
     * Check if this table has a given column name. 
     * 
     * Used to verify datablocks.
     * 
     * @param string $name The table column name
     * @return bool
     */
    public function hasColumnName($name) {
        foreach ($this->cols as $o) {
            if ($o->Field == $name)
                return true;
        }
        return false;
    }

    private $database;
    private $name;
    private $cols = array();
    private $primary_column;

}

?>
