<?php

require 'Filter.php';
require 'Selector.php';
require 'TableDescriptor.php';
require 'DataDescriptor.php';

/**
 * A table filter contains the algorithm to go through a given table, apply
 * selectors and finally the filter. It uses a given data descriptor to request
 * the creation of additional data blocks as needed.
 * 
 * The database table is not changes with this class. To change the table, use 
 * the Ssr_Execute class.
 *
 * @category    Ssr
 * @package     Ssr
 * @subpackage  TableFilter
 * @version     Release: 1.0
 * @since       Class available since Release 1.0
 * @author      Benjamin Sommer <developer@benjaminsommer.com>
 * @uses        Ssr_TablePartialBackup, Ssr_Filter, Ssr_Selector, 
 *              Ssr_TableDescriptor
 */
class Ssr_TableFilter {

    /**
     * Initialize the table filer
     * 
     * @param Ssr_TableDescriptor   $tableDescriptor
     * @param mysqli                $db
     * @param Ssr_DataDescriptor    $backupDataDescriptor 
     */
    public function __construct($tableDescriptor, $db, $backupDataDescriptor) {
        $this->table_desc = $tableDescriptor;
        $this->db = $db;
        $this->datadesc = $backupDataDescriptor;
    }

    /**
     * Executes a selection and filter on a given database table. 
     * No changes are made to the table. Use Ssr_Execute to actually change the
     * table given the data blocks (which are calculated with this method)
     */
    public function exec() {
        $query = $this->mkSelectSQL();
        $result = $this->db->query($query);
        if ($result) {
            $datablock = $this->datadesc->createDatablock();
            while ($rs = $result->fetch_object()) {
                foreach ($this->regexSelectors as $selector) {
                    if (!$selector->matches($rs))
                        continue 2;
                }

                $post = array();
                foreach ($this->filter as $e) {
                    if ($e->filter($rs)) {
                        $col = $e->getColumnName();
                        $post[$col] = $e->getResult();
                    }
                }

                foreach ($post as $colName => $postContent) {
                    $rowid = $this->table_desc->getResultsetKey($rs);
                    if (!empty($rowid)) {
                        if (!$datablock->hasSpace()) {
                            $datablock->save();
                            $this->datadesc->updateFromDatablock($datablock);
                            $datablock = $this->datadesc->createDatablock();
                        }
                        $preContent = $rs->$colName;
                        $datablock->pushCell($rowid, $colName, $preContent, $postContent);
                    } else {
                        continue 2;
                    }
                }
            }
            $datablock->save();
            $this->datadesc->updateFromDatablock($datablock);
            $this->datadesc->readTableFilter($this);
            $this->datadesc->save();
        } else {
            Ssr_Warning::push(100, "No results for filter process `{$this->datadesc->getName()}`");
        }
    }

    /**
     * Generate a SQL select statement based on non-regexp selectors
     * 
     * All selectors are applied with the sql AND operator.
     * 
     * @return string 
     */
    private function mkSelectSQL() {
        $t = array();
        $fields = array();
        foreach ($this->stringSelectors as $obj) {
            $t[] = "`{$obj->getColumnName()}` LIKE '%{$this->db->real_escape_string($obj->getPattern())}%'";
            $fields[$obj->getColumnName()] = "`{$obj->getColumnName()}`";
        }
        foreach ($this->filter as $obj) {
            $fields[$obj->getColumnName()] = "`{$obj->getColumnName()}`";
        }
        $pri = $this->table_desc->getPrimaryColumnName();
        $fields[$pri] = "`$pri`";

        $q = "SELECT ";
        if (!empty($fields))
            $q .= implode(', ', $fields);
        else
            $q .= '*';
        $q .= " FROM `{$this->table_desc->getDatabaseName()}`.`{$this->table_desc->getTableName()}` ";
        if (!empty($t)) {
            $q .= ' WHERE ' . implode(' AND ', $t);
        }
        return $q;
    }

    /**
     * Push a selector to the table selector stack
     * 
     * Selectors are used to narrow possible result sets returned from database, 
     * upon which are applied the filter.
     * 
     * @param Ssr_Selector $selector 
     */
    public function pushSelector($selector) {
        if ($this->table_desc->hasColumnName($selector->getColumnName())) {
            if ($selector->isRegexp())
                $this->regexSelectors[] = $selector;
            else
                $this->stringSelectors[] = $selector;
        } else
            Ssr_Error::push(400, "Undefined column {$selector->getColumnName()} for selector");
    }

    /**
     * Push a filter to the table filter stack
     * 
     * Filters are used to search and replace data within table cells.
     * 
     * @param Ssr_Filter $filter 
     */
    public function pushFilter($filter) {
        if ($this->table_desc->hasColumnName($filter->getColumnName()))
            $this->filter[] = $filter;
        else
            Ssr_Error::push(400, "Undefined column {$filter->getColumnName()} for filter");
    }

    /**
     * Count number of selectors
     * @return int
     */
    public function countSelectors() {
        return sizeof($this->stringSelectors) + sizeof($this->regexSelectors);
    }

    /**
     * Get selector by index
     * @param int $index
     * @return Ssr_Selector
     */
    public function getSelector($index) {
        $s = sizeof($this->stringSelectors);
        if ($index < $s)
            return $this->stringSelectors[$index];
        else
            return $this->regexSelectors[$index - $s];
    }

    /**
     * Get an array of selector instances
     * @return array
     */
    public function getSelectors() {
        $t = $this->stringSelectors;
        foreach ($this->regexSelectors as $s)
            $t[] = $s;
        return $t;
    }

    /**
     * Get an array of filter objects
     * @return array
     */
    public function getFilters() {
        return $this->filter;
    }

    /**
     * Count number of attached filter
     * @return int
     */
    public function countFilter() {
        return sizeof($this->filter);
    }

    /**
     * Get filter by index
     * @param int $index
     * @return Ssr_Filter
     */
    public function getFilter($index) {
        return $this->filter[$index];
    }

    /**
     * Get the table descriptor
     * @return Ssr_TableDescriptor
     */
    public function getTableDescriptor() {
        return $this->table_desc;
    }

    /**
     *
     * @var Ssr_TableDescriptor
     */
    private $table_desc;
    private $stringSelectors = array();
    private $regexSelectors = array();
    private $filter = array();

    /**
     *
     * @var mysqli
     */
    private $db;

    /**
     * @var Ssr_DataDescriptor
     */
    private $datadesc;

}

?>
