<?php

/**
 * Extend WP options functionality
 * 
 * When getting options, this class caches the result in memory to minimalize 
 * database queries and unserializations. To make full use of caching, option 
 * fragmentation should be avoided whenever possible, i.e. options usually 
 * fetched together, options with similar meaning (for a particular package or 
 * subpackage) should be grouped together in an array.
 * 
 * @category    Ssr
 * @package     Ssr
 * @subpackage  Option
 * @version     Release: 2.10
 * @since       Class available since Release 2
 * @author      Benjamin Sommer <developer@benjaminsommer.com>
 */
class Ssr_Option 
{
    
    /**
     * Retrieve option value based on name of option and package name.  
     * 
     * If the option does not exist or does not have a value, then the return 
     * value will be null. This is useful to check whether you need to install 
     * an option and is commonly used during installation of plugin options and 
     * to test whether upgrading is required.  
     * If the option was serialized then it will be unserialized when 
     * it is returned.
     *
     * @param   string  $optionName     Name of option to retrieve.
     * @param   string  $packageName    Package name to which the option name 
     *                                  belongs. Expected to not be SQL-escaped.
     * @return  mixed                   Value set for the option, or null if option
     *                                  does not exist
     */
    public static function get($optionName, $packageName) {
        if (self::isCached($optionName, $packageName)) {
            return self::getCache($optionName, $packageName);
        }
        $packageValues = get_option($packageName);
        if ($packageValues !== false)
            self::doCache($packageName, $packageValues);
        if (isset($packageValues[$optionName]))
            return $packageValues[$optionName];
        return null;
    }
    
    /**
     *
     * Retrieve option value based on option name, option sub name and package name.
     * 
     * @uses            Ssr_Option::get
     * @param   string  $optionName     Name of option to retrieve.
     * @param   string  $optionSubname  Subname of option to retrieve.
     * @param   string  $packageName    Package name to which the option name 
     *                                  belongs. Expected to not be SQL-escaped.
     * @return  mixed                   Value set for the option, or null if option
     *                                  does not exist
     */
    public static function getSub($optionName, $optionSubname, $packageName) {
        $option = self::get($optionName, $packageName);
        return isset($option[$optionSubname]) ? $option[$optionSubname] : null;
    }
    

    /**
     * Add a new option for a package name. 
     * 
     * You do not need to serialize values. If the value needs to be serialized, 
     * then it will be serialized before it is inserted into the database. 
     * Remember, resources can not be serialized or added as an option.  
     * 
     * You can create options without values and then add values later. 
     * Checks whether the option has already been added, and updates it if required, and
     * does check that you aren't adding a protected WordPress option (for WP as package name). 
     * 
     * Care should be taken to not name packages the same as the ones which are 
     * protected.
     *
     * @param   string  $optionName     Option name for the given package name.
     * @param   mixed   $optionValue    Optional. Option value, can be anything.
     * @param   string  $packageName    Enclosing package name. Expected to not be SQL-escaped.
     * @return  bool                    False if option has not been updated
     */
    public static function set($optionName, $optionValue, $packageName) {
        self::get($optionName, $packageName);
        if (self::isCached($optionName, $packageName) && self::getCache($optionName, $packageName) == $optionValue)
            return true;
        $update = isset(self::$_cache[$packageName]);
        self::$_cache[$packageName][$optionName] = $optionValue;
        if ($update)
            return update_option($packageName, self::$_cache[$packageName]);
        else
            add_option($packageName, self::$_cache[$packageName], '', false);
        return true;
    }

    /**
     * Check if the option name is cached for the enclosing package.
     *
     * @param   string  $optionName     Option name for the given package name.
     * @param   string  $packageName    Enclosing package name. Expected to not be SQL-escaped.
     * @return  bool                    True if cached
     */
    private static function isCached($optionName, $packageName) {
        return isset(self::$_cache[$packageName][$optionName]);
    }

    /**
     * Cache all options of a package. 
     * 
     * Caching is used to improve performance, because database queries will be kept low.
     *
     * @param string    $packageName       The package name. Expected to not be SQL-escaped.
     * @param mixed     $packageValues     Package option values, can be anything; usually an array.
     */
    private static function doCache($packageName, $packageValues) {
        self::$_cache[$packageName] = $packageValues;
    }

    /**
     * Get the cache for an option of a package.
     *
     * @param   string  $optionName     Option name for the given package name.
     * @param   string  $packageName    The package name. Expected to not be SQL-escaped.
     * @return  mixed                   The cached option value
     */
    private static function getCache($optionName, $packageName) {
        return self::$_cache[$packageName][$optionName];
    }

    /**
     * The cache of package options
     *
     * @var array 
     */
    private static $_cache = array();
}

?>
