<?php

/**
 * This class creates the UI so that the user can create, review and execute
 * table filter. This is the main search and replace functionality on the user side.
 * The safety comes from the core, the SSR API.
 *
 * @category    Ssr
 * @package     Ssr_UI
 * @subpackage  TableFilter
 * @version     Release: 1.1
 * @since       Class available since Release 1.0
 * @author      Benjamin Sommer <developer@benjaminsommer.com>
 */
class Ssr_UI_TableFilter {

    /**
     * Create a new UI to create table filter. Used as callback.
     * @return Ssr_UI_TableFilter 
     */
    public static function create() {
        return new Ssr_UI_TableFilter();
    }

    /**
     * Initialize and create the UI to search and replace in database tables.
     * @uses    Ssr_UI_AdminPanel
     */
    public function __construct() {
        $this->dbname = Ssr_UI_AdminPanel::getMysqliDbName();
        $this->db = Ssr_UI_AdminPanel::getMysqliConnection();

        if (!isset($_GET['table']) || empty($_GET['table'])) {
            $this->printChooseTable();
        } else {
            if (isset($_POST) && !empty($_POST) &&
                    isset($_POST['name']) && !empty($_POST['name']) &&
                    isset($_POST['desc']) && !empty($_POST['desc']) &&
                    isset($_POST['filterfrom']) && !empty($_POST['filterfrom']) &&
                    isset($_POST['filterto']) && !empty($_POST['filterto'])) {
                $this->runSimulate();
            } else if (isset($_GET['simid']) && isset($_GET['blockid'])) {
                if (isset($_GET['execute'])) {
                    $this->executeBlock();
                    unset($_GET['execute']);
                }
                $this->printSimulate();
            } else
                $this->printDefineFilter();
        }
    }

    /**
     * Print a list of available database tables for the current connection
     * and database name.
     * @uses    Ssr_TableDescriptor
     */
    private function printChooseTable() {
        echo '<h3>';
        echo __('Step 1: Select a Database Table', 'ssr');
        echo '</h3>';
        $tables = $this->getTableNames();
        echo '<table class="widefat" style="width:700px">
        <thead>
            <tr>
                <th style="width:200px">' . __('Table Name', 'ssr') . '</th>
                <th style="width:100px">' . __('Table Rows', 'ssr') . '</th>
                <th>' . __('Usage Hint', 'ssr') . '</th>
                <th style="width:100px">' . __('Commands', 'ssr') . '</th>
            </tr>
        </thead>
        <tfoot>
            <tr>
                <th>' . __('Table Name', 'ssr') . '</th>
                <th>' . __('Table Rows', 'ssr') . '</th>
                <th>' . __('Usage Hint', 'ssr') . '</th>
                <th>' . __('Commands', 'ssr') . '</th>
            </tr>
        </tfoot>
        <tbody>';

        foreach ($tables as $prop) {
            $name = $prop['name'];
            $tabledesc = new Ssr_TableDescriptor($name, $this->db);
            $usage = __('Not Available', 'ssr');
            echo "<tr><td>$name</td>
                <td>{$prop['rows']}</td>
                <td>$usage</td>
                <td><a href=\"?" . http_build_query(array_merge($_GET, array('table' => $name))) . "\">
                    " . __('Select', 'ssr') . "
                    </a></td>
                </tr>";
        }

        echo '</tbody>
        </table>';
    }

    /**
     * Print the UI to create the actual filter and selector, to define other properties
     * (like name, description, blocksize). 
     * The user has the option to preview the filter (first 10-50 results) without
     * any changes using ajax.
     */
    private function printDefineFilter() {
        $tabledesc = new Ssr_TableDescriptor($_GET['table'], $this->db);

        ini_set('max_execution_time', 5 * 60);
        $max_exec_time = ini_get('max_execution_time');

        echo '<form action="" method="post">';
        echo '<input type="hidden" name="table" value="' . $_GET['table'] . '"  />';

        echo '<div style="float:right">';
        echo '<input type="button" value="' . __('Preview Selection', 'ssr') . '" class="button-secondary"
                    onclick="ssr_preview_selection(this)"/> ';
        echo '<input type="submit" value="' . __('Simulate', 'ssr') . '" class="button-primary" />';
        echo '</div>';

        echo "<h3>";
        echo __('Step 2: Define Table Filter', 'ssr');
        echo "</h3>";

        echo '<table><tr valign="top"><td>';
        echo '<table class="widefat" style="width:750px">
            <thead>
                <tr>
                    <th style="width:30px">' . __('Key', 'ssr') . '</th>
                    <th style="width:170px">' . __('Column Name', 'ssr') . '</th>
                    <th style="width:200px">' . __('Selector', 'ssr') . '</th>
                    <th style="width:350px">' . __('Filter', 'ssr') . '</th>
                </tr>
            </thead>
            <tfoot>
                <tr>
                    <th>' . __('Key', 'ssr') . '</th>
                    <th>' . __('Column Name', 'ssr') . '</th>
                    <th>' . __('Selector', 'ssr') . '</th>
                    <th>' . __('Filter', 'ssr') . '</th>
                </tr>
            </tfoot>
            <tbody>';

        $numCols = $tabledesc->size();
        $pricol = $tabledesc->getPrimaryColumnName();
        for ($i = 0; $i < $numCols; $i++) {
            $colname = $tabledesc->getColumnName($i);
            echo "<tr><td>{$tabledesc->getColumnKey($i)}</td>
                    <td>$colname</td>
                    <td><div><input type=\"button\" value=\"" . __('Add Selector', 'ssr') . "\"
                                onclick=\"ssr_add_selector('$colname',this)\"
                                class=\"button-secondary\" /></div></td>";
            if ($colname != $pricol)
                echo "<td><div><input type=\"button\" value=\"" . __('Add Filter', 'ssr') . "\"
                                onclick=\"ssr_add_filter('$colname',this)\"
                                class=\"button-secondary\" /></div></td>";
            else
                echo "<td></td>";
            echo "</tr>";
        }

        echo '</tbody>
            </table><br />';
        echo '</td><td>';
        echo '<table class="form-table">
                    <tbody>
                        <tr valign="top">
                            <td scope="row"><strong>' . __('Table Name', 'ssr') . '</strong></th>
                            <td><strong>' . $tabledesc->getTableName() . '</strong></td>
                        </tr>
                        <tr valign="top">
                            <th scope="row"><label for="ssr-fname"><strong>' . __('Filter Name', 'ssr') . '*</strong></label></th>
                            <td><input id="ssr-fname" class="regular-text" type="text" value="" name="name"></td>
                        </tr>
                        <tr valign="top">
                            <th scope="row"><label for="ssr-desc"><strong>' . __('Filter Description', 'ssr') . '*</strong></label></th>
                            <td><textarea id="ssr-desc" name="desc" style="width:500px;height:100px"></textarea></td>
                        </tr>
                        <tr valign="top">
                            <th scope="row">' . __('Blocksize', 'ssr') . '</th>
                            <td><select name="blocksize">
                                    <option value="100">' . sprintf(__('%s Cells', 'ssr'), 100) . '</option>
                                    <option value="250" selected="selected">' . sprintf(__('%s Cells', 'ssr'), 250) . '</option>
                                    <option value="500">' . sprintf(__('%s Cells', 'ssr'), 500) . '</option>
                                    <option value="1000">' . sprintf(__('%s Cells', 'ssr'), 1000) . '</option></select></td>
                        </tr>
                        <tr valign="top">
                            <th scope="row">' . __('Maximum Execution Time', 'ssr') . '</th>
                            <td>' . sprintf(__('%d minutes', 'ssr'), $max_exec_time / 60) . '</td>
                        </tr>
                    </tbody>
                 </table>';
        echo '<br />';
        echo '</td></tr></table>';
        echo '</form>';
        echo '<div id="ssr-result-preview"></div>';
    }

    /**
     * The main function to create a simulation, i.e. to create a data descriptor and its
     * data blocks for given previous options. No changes to the database are made yet.
     * @uses    Ssr_TableDescriptor, Ssr_DataDescriptor, Ssr_TableFilter
     *          Ssr_Selector, Ssr_Filter
     */
    private function runSimulate() {
        $tabledesc = new Ssr_TableDescriptor($_GET['table'], $this->db, $this->dbname);
        $datadesc = new Ssr_DataDescriptor($_POST['name'], $_POST['desc'], SSR_PATH . 'data/simulate/', $tabledesc);
        $tablefilter = new Ssr_TableFilter($tabledesc, $this->db, $datadesc);
        if (isset($_POST['blocksize']) && !empty($_POST['blocksize']))
            $datadesc->setBlocksize((int) $_POST['blocksize']);

        if (isset($_POST['selector'])) {
            foreach ($_POST['selector'] as $column => $list) {
                foreach ($list as $pattern) {
                    $selector = new Ssr_Selector($column, $pattern);
                    $tablefilter->pushSelector($selector);
                }
            }
        }
        if (isset($_POST['filterfrom'])) {
            foreach ($_POST['filterfrom'] as $column => $list) {
                foreach ($list as $k => $search_pattern) {
                    if (isset($_POST['filterto'][$column][$k])) {
                        $replace_pattern = $_POST['filterto'][$column][$k];
                        $filter = new Ssr_Filter($column, $search_pattern, $replace_pattern);
                        $tablefilter->pushFilter($filter);
                    }
                }
            }
        }

        $tablefilter->exec();

        $_GET['simid'] = $datadesc->getId();
        $_GET['blockid'] = 0;
        echo '<a href="?' . http_build_query($_GET) . '">' . __('Click Here to Review Simulation', 'ssr') . '...</a>';
    }

    /**
     * The UI to print the simulation, i.e. to review all created data blocks and
     * execute them as wished.
     * @uses    Ssr_DataDescriptor, Ssr_Datablock, Ssr_DatablockViewer
     */
    private function printSimulate() {
        $viewer = 'TableViewer';
        if(isset($_GET['viewer']))
            $viewer = $_GET['viewer'];
        require SSR_PATH . "Ssr/Api/Datablock$viewer.php";

        $simid = $_GET['simid'];
        $blockid = $_GET['blockid'];
        if (isset($_GET['cellstart']))
            $cellstart = $_GET['cellstart'];
        else
            $cellstart = 0;
        if (isset($_GET['cellend']))
            $cellend = $_GET['cellend'];
        else
            $cellend = 100;

        $execblocks = array();
        if (isset($_GET['execblocks']))
            $execblocks = explode(',', $_GET['execblocks']);

        $file = SSR_PATH . "data/simulate/$simid.descriptor.xml";
        $datadesc = Ssr_DataDescriptor::createFromFile($file);
        if (sizeof($execblocks) == $datadesc->countBlocks()) {
            echo '<div id="message" class="updated"><p>';
            printf(__('<strong>Executed all blocks</strong> for task #%s.'), $simid);
            _e('See <em>Reports</em> starting with this task id, or <em>Previous Operations</em> (review, undo).', 'ssr');
            echo '</p></div>';
            return;
        } else if (in_array($blockid, $execblocks)) {
            for ($i = 0; $i < $datadesc->countBlocks(); $i++) {
                if (!in_array($i, $execblocks)) {
                    $blockid = $_GET['blockid'] = $i;
                    break;
                }
            }
        }

        $datablock = Ssr_Datablock::createFromFile($datadesc->getBlockPath($blockid));
        if($viewer == 'TableViewer')
            $blockviewer = new Ssr_DatablockTableViewer($datablock, $this->db);
        else $blockviewer = new Ssr_DatablockListViewer($datablock, $this->db);
        $blockviewer->setTableclass('widefat');

        echo "<div style=\"float:right\">";
        echo '<form action="" method="get">';

        echo "<small>";
        printf(__('%1$s/%2$s Table Cells to be updated', 'ssr'), $datablock->countCells(), $datadesc->countCells());
        echo '</small>';
        echo ' | ';

        foreach ($_GET as $k => $v) {
            if ($k != 'blockid' && $k != 'cellend' && $k != 'cellstart')
                echo "<input type=\"hidden\" name=\"$k\" value=\"$v\" />";
        }
        _e('Results', 'ssr');
        echo ": <input type=\"text\" value=\"$cellstart\" name=\"cellstart\" size=\"7\" />
                    -> <input type=\"text\" value=\"$cellend\" name=\"cellend\" size=\"7\" />";
        echo ' | <select name="blockid">';
            for ($i = 0; $i < $datadesc->countBlocks(); $i++) {
                if (!in_array($i, $execblocks))
                    echo "<option value=\"$i\" " . selected($i, $blockid, false) . ">" . sprintf(__('Block #%s', 'ssr'), $i) . "</option>";
            }
            echo '</select> ';
        echo ' | <select name="viewer">';
            $viewers = array();
            $viewers['TableViewer'] = __('Table','ssr');
            $viewers['ListViewer'] = __('List','ssr');
            echo "<option>" . __('View As','ssr') . "</option>";
            foreach($viewers as $k=>$v)
                echo "<option value=\"$k\" " . selected($k, $viewer, false) . ">$v</option>";
        echo '</select> ';
        echo ' <input type="submit" value="' . __('Update View', 'ssr') . '" class="button-secondary" /> || ';

        echo '<input type="submit" value="' . __('Execute Block', 'ssr') . '" class="button-primary" name="execute" />';
        echo '</form>';
        echo "</div>";
        echo "<h3>";
        _e('Step 3: Review Simulation', 'ssr');
        echo "</h3>";

        echo $blockviewer->view($cellstart, $cellend);
    }

    /**
     * Execute a data block and actually change the database tables. This operation
     * can be undone by backward/forward execution. Reports and past operations will be 
     * created.
     * @uses    Ssr_DataDescriptor, Ssr_Execute
     */
    private function executeBlock() {
        require_once SSR_PATH . 'Ssr/Api/Execute.php';

        $simid = $_GET['simid'];
        $blockId = $_GET['blockid'];
        $file = SSR_PATH . "data/simulate/$simid.descriptor.xml";
        $datadesc = Ssr_DataDescriptor::createFromFile($file);
        $execute = new Ssr_Execute($datadesc, $this->db, SSR_PATH . 'data/exec/', SSR_PATH . 'data/reports/');
        $execute->exec($blockId);
        $report = $execute->getReportPath();

        $t = array();
        if (isset($_GET['execblocks']))
            $t = explode(',', $_GET['execblocks']);
        $t[] = $blockId;
        $_GET['execblocks'] = implode(',', $t);

        if ($execute->countCellsFail() == 0) {
            echo '<div id="message" class="updated"><p>';
            printf(__('<strong>Executed block #%1$s</strong> from task #%2$s.', 'ssr'), $blockId, $simid);
        } else {
            echo '<div id="message" class="error"><p>';
            printf(__('<strong>Executed %1$s/%2$s</strong> in block #%3$s from task #%4$s.', 'ssr'), $execute->countCellsOk(), $execute->countCellsFail(), $blockId, $simid);
        }
        echo ' ';
        printf(__('See detailed report (%s)', 'ssr'), basename($report));
        echo '</p></div>';
    }

    /**
     * Get an array of available table names
     * @return array An array with {[name][rows]} 
     */
    private function getTableNames() {
        $tables = array();
        $sql = "SELECT TABLE_NAME, TABLE_ROWS FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA LIKE '{$this->dbname}'";
        $result = $this->db->query($sql);
        if ($result) {
            while ($o = $result->fetch_object()) {
                $t = array();
                $t['name'] = $o->TABLE_NAME;
                $t['rows'] = $o->TABLE_ROWS;
                $tables[] = $t;
            }
        }
        return $tables;
    }

    /**
     * Preview a selection/table filter. This method is used for AJAX.
     */
    public static function previewSelection() {
        require_once 'AdminPanel.php';
        require SSR_API . 'DatablockListViewer.php';
        $db = Ssr_UI_AdminPanel::getMysqliConnection();
        $dbname = Ssr_UI_AdminPanel::getMysqliDbName();
        parse_str(html_entity_decode($_POST['form']), $post);
        $limit = 10;

        $tabledesc = new Ssr_TableDescriptor($post['table'], $db, $dbname);
        $datadesc = new Ssr_DataDescriptor($post['name'], $post['desc'], SSR_DATA . 'simulate/', $tabledesc);
        $datadesc->storeInMemory(true);
        $tablefilter = new Ssr_TableFilter($tabledesc, $db, $datadesc);
        if (isset($post['blocksize']) && !empty($post['blocksize']))
            $datadesc->setBlocksize((int) $post['blocksize']);

        if (isset($post['selector'])) {
            foreach ($post['selector'] as $column => $list) {
                foreach ($list as $pattern) {
                    $selector = new Ssr_Selector($column, $pattern);
                    $tablefilter->pushSelector($selector);
                }
            }
        }
        if (isset($post['filterfrom'])) {
            foreach ($post['filterfrom'] as $column => $list) {
                foreach ($list as $k => $search_pattern) {
                    if (isset($post['filterto'][$column][$k])) {
                        $replace_pattern = $post['filterto'][$column][$k];
                        $filter = new Ssr_Filter($column, $search_pattern, $replace_pattern);
                        $tablefilter->pushFilter($filter);
                    }
                }
            }
        }
        $tablefilter->exec();
        $block = $datadesc->getBlock(0);
        $viewer = new Ssr_DatablockListViewer($block, $db);
        $viewer->setTableclass('widefat');
        echo $viewer->view(0, $limit);
        Ssr_Error::printStack('<br />');
        Ssr_Warning::printStack('<br />');
    }

    /**
     *
     * @var mysqli
     */
    private $db;
    private $dbname;

}

?>
